! Monte Carlo Event Generator for DI
! of He 
      PROGRAM MCEG_DI
! Uncomment if you use Intel Fortran Compilers      
!	USE IFLPORT
	IMPLICIT NONE
	REAL*8 MP,VI,EPS,V_C,PI,ZP,EI
	REAL*8 RY,DELTAE,EBI,QMIN,QP,PHIQ,QP1,QP2
	REAL*8 EBIHE,EBIHEP,QZ1,QZ2,DELTAHE,DELTAHEP
	REAL*8 FDCSDITS1U,FDCSDITS1,FDCSDITS1MAX,FDCSDITS1N
	REAL*8 FDCSHEU,FDCSHEN,FDCSHEMAX,FDCSDITS2
	REAL*8 FDCSHEPU,FDCSHEPN,FDCSHEPMAX
	REAL*8 K1X,K1Y,K1Z,K1,QMAX,U,U1,U2
	REAL*8 K2X,K2Y,K2Z,K2,K12,GAMOV,SOMM,ZEFF12
	REAL*8 K1XMIN,K1XMAX,K1YMIN,K1YMAX,K1ZMIN,K1ZMAX
	REAL*8 K2XMIN,K2XMAX,K2YMIN,K2YMAX,K2ZMIN,K2ZMAX
	INTEGER THEORY,MC,MCPOINTS,SCREEN,EEMODEL
	INTEGER ISEED,ITASK,STEP
	CHARACTER*3 TASK_ID
	EXTERNAL FDCSDITS1,FDCSDITS2
	PARAMETER(PI=3.1415926535897932D0,MP=1836D0,EPS=1D-6)
	PARAMETER(V_C=137D0,RY=27.21D0)
! File load
	OPEN(UNIT=9,FILE='di.dat',STATUS='UNKNOWN')
	READ(9,*) ZP
	READ(9,*) EI
	READ(9,*) THEORY
	READ(9,*) EEMODEL
	READ(9,*) K1XMIN, K1XMAX
	READ(9,*) K1YMIN, K1YMAX
	READ(9,*) K1ZMIN, K1ZMAX	  
	READ(9,*) K2XMIN, K2XMAX
	READ(9,*) K2YMIN, K2YMAX
	READ(9,*) K2ZMIN, K2ZMAX	  
	READ(9,*) QMAX
	READ(9,*) MCPOINTS
	READ(9,*) SCREEN
	READ(9,*) ISEED
	CLOSE(9)	
! INITIAL VELOCITY
      VI=V_C*DSQRT((1D0-(EI*1000D0/(RY*MP*V_C**2)+1D0)**(-2)))
! Parallel implementation
	IF(ISEED.EQ.0)THEN
! SGE 
!	CALL GETENV("SGE_TASK_ID",TASK_ID)
! TORQUE
!	CALL GETENV("PBS_ARRAYID",TASK_ID)
	READ(TASK_ID,'(I3)') ITASK
	SCREEN=0
	ENDIF
! Test if only screen or file output
	IF(SCREEN.EQ.0)THEN
	IF(ISEED.EQ.0)THEN
! Check if your system needs the full path		
	OPEN(7,FILE='results.dat.'//TASK_ID,
     *  STATUS='UNKNOWN')
	ISEED=ITASK
	ELSE
	OPEN(7,FILE='results.dat',
     *  STATUS='UNKNOWN')
	ENDIF
	ENDIF
! TS-1 Calculations
	IF(THEORY.EQ.0)THEN
! DI energy of He
	EBI=2.904D0	
! Some initialization
	FDCSDITS1U=0D0
	FDCSDITS1N=0D0
	FDCSDITS1MAX=0D0
	PHIQ=0D0
! Maximum value of FDCS calculation
	CALL FDCSDITS1M(ZP,VI,QMAX,FDCSDITS1MAX)
! MC main loop
	DO MC=1,MCPOINTS
	CALL RND(ISEED,0D0,1D0,U)
	CALL RND(ISEED,0D0,QMAX,QP)
	CALL RND(ISEED,K1XMIN,K1XMAX,K1X)
	CALL RND(ISEED,K1YMIN,K1YMAX,K1Y)
	CALL RND(ISEED,K1ZMIN,K1ZMAX,K1Z)
	CALL RND(ISEED,K2XMIN,K2XMAX,K2X)
	CALL RND(ISEED,K2YMIN,K2YMAX,K2Y)
	CALL RND(ISEED,K2ZMIN,K2ZMAX,K2Z)
	K1=DSQRT(K1X**2+K1Y**2+K1Z**2)
	K2=DSQRT(K2X**2+K2Y**2+K2Z**2)
	DELTAE=(EBI+(K1**2)/2D0+(K2**2)/2D0)
! QMIN is not saved in the output file. 
! The calculation is just for completeness	
	QMIN=DELTAE/VI
	FDCSDITS1U=FDCSDITS1(ZP,VI,QP,PHIQ,
     * K1X,K1Y,K1Z,K2X,K2Y,K2Z)
! Relative momentum
	K12=DSQRT((K1X-K2X)**2+(K1Y-K2Y)**2+(K1Z-K2Z)**2)
	IF(K12.LT.1D-3)THEN
	K12=1D-3
	ENDIF
! e-e Models
! Uncorrelated
	IF(EEMODEL.EQ.0)THEN
	GAMOV=1D0
	ELSE
! Correlated models
! Static screening
	IF(EEMODEL.EQ.1)THEN
	ZEFF12=1D0
	SOMM=2D0*PI*ZEFF12/K12
	GAMOV=SOMM/(DEXP(SOMM)-1D0)
	ELSE
! Dynamic screening
	IF(EEMODEL.EQ.2)THEN
	ZEFF12=1D0-(K12**2)/((K1+K2)**2)
	SOMM=2D0*PI*ZEFF12/K12
	GAMOV=SOMM/(DEXP(SOMM)-1D0)		
	ELSE
	WRITE(*,*) 'e-e model not supported'
	STOP
	ENDIF
	ENDIF
	ENDIF		
	FDCSDITS1U=GAMOV*FDCSDITS1U
! Normalization
	FDCSDITS1N=FDCSDITS1U/FDCSDITS1MAX
! MC test
	IF(U.LT.FDCSDITS1N)THEN
! Only file selection (forced when parallel implementation is used)
	IF(SCREEN.EQ.0)THEN
	WRITE(7,100) K1X,K1Y,K1Z,K2X,K2Y,K2Z,QP
	ELSE
! Only screen
	IF(SCREEN.EQ.1)THEN
	WRITE(*,100) K1X,K1Y,K1Z,K2X,K2Y,K2Z,QP
	ELSE
	WRITE(*,*) 'Option not supported'
	STOP
	ENDIF
	ENDIF
	ENDIF
	ENDDO
	ELSE
	
! TS-2 Models
! FBA
	IF(THEORY.EQ.1)THEN
! First Step: SI of He
	EBIHE=0.91D0	
	STEP=1
	FDCSHEU=0D0
	FDCSHEN=0D0
	FDCSHEMAX=0D0
	PHIQ=0D0
! Maximum value of FDCS calculation
	CALL FDCSTS2M(ZP,VI,QMAX,STEP,THEORY,FDCSHEMAX)
! Second Step: SI of He+
	EBIHEP=2D0
	STEP=2	
	FDCSHEPU=0D0
	FDCSHEPN=0D0
	FDCSHEPMAX=0D0
! Maximum value of FDCS calculation
	CALL FDCSTS2M(ZP,VI,QMAX,STEP,THEORY,FDCSHEPMAX)
! MC main loop
	DO MC=1,MCPOINTS
	CALL RND(ISEED,0D0,1D0,U1)
	CALL RND(ISEED,0D0,1D0,U2)
	CALL RND(ISEED,0D0,QMAX,QP1)
	CALL RND(ISEED,0D0,QMAX,QP2)
	CALL RND(ISEED,K1XMIN,K1XMAX,K1X)
	CALL RND(ISEED,K1YMIN,K1YMAX,K1Y)
	CALL RND(ISEED,K1ZMIN,K1ZMAX,K1Z)
	CALL RND(ISEED,K2XMIN,K2XMAX,K2X)
	CALL RND(ISEED,K2YMIN,K2YMAX,K2Y)
	CALL RND(ISEED,K2ZMIN,K2ZMAX,K2Z)
	K1=DSQRT(K1X**2+K1Y**2+K1Z**2)
	K2=DSQRT(K2X**2+K2Y**2+K2Z**2)
	DELTAHE=EBIHE+K1**2/2D0
	DELTAHEP=EBIHEP+K2**2/2D0
! QZ1 and QZ2 are not used but are calculated for completeness	
	QZ1=DELTAHE/VI
	QZ2=DELTAHEP/VI
! FDCS calculations
! SI of He
	STEP=1
	FDCSHEU=FDCSDITS2(ZP,VI,THEORY,STEP,
     *   	QP1,PHIQ,K1X,K1Y,K1Z)
      FDCSHEN=FDCSHEU/FDCSHEMAX
! SI of He+      
	STEP=2
	FDCSHEPU=FDCSDITS2(ZP,VI,THEORY,STEP,
     *   	QP2,PHIQ,K2X,K2Y,K2Z)    
      FDCSHEPN=FDCSHEPU/FDCSHEPMAX
      IF(FDCSHEN.GT.U1.AND.FDCSHEPN.GT.U2)THEN
! Only file selection (forced when parallel implementation is used)
	IF(SCREEN.EQ.0)THEN
	WRITE(7,110) K1X,K1Y,K1Z,K2X,K2Y,K2Z,QP1,QP2
	ELSE
! Only screen
	IF(SCREEN.EQ.1)THEN
	WRITE(*,110) K1X,K1Y,K1Z,K2X,K2Y,K2Z,QP1,QP2
	ELSE
	WRITE(*,*) 'Option not supported'
	STOP
	ENDIF
	ENDIF
	ENDIF
      ENDDO
	ELSE
	
! CDW-EIS
	IF(THEORY.EQ.2)THEN
! Maximum calculations	
! First Step: SI of He	
	STEP=1
	FDCSHEU=0D0
	FDCSHEN=0D0
	FDCSHEMAX=0D0
! Maximum value of FDCS calculation
	CALL FDCSTS2M(ZP,VI,QMAX,STEP,THEORY,FDCSHEMAX)
! Second Step: SI of He+
	STEP=2	
	FDCSHEPU=0D0
	FDCSHEPN=0D0
	FDCSHEPMAX=0D0
! Maximum value of FDCS calculation
	CALL FDCSTS2M(ZP,VI,QMAX,STEP,THEORY,FDCSHEPMAX)
! MC main loop
	DO MC=1,MCPOINTS
	CALL RND(ISEED,0D0,1D0,U1)
	CALL RND(ISEED,0D0,1D0,U2)
	CALL RND(ISEED,0D0,QMAX,QP1)
	CALL RND(ISEED,0D0,QMAX,QP2)
	CALL RND(ISEED,K1XMIN,K1XMAX,K1X)
	CALL RND(ISEED,K1YMIN,K1YMAX,K1Y)
	CALL RND(ISEED,K1ZMIN,K1ZMAX,K1Z)
	CALL RND(ISEED,K2XMIN,K2XMAX,K2X)
	CALL RND(ISEED,K2YMIN,K2YMAX,K2Y)
	CALL RND(ISEED,K2ZMIN,K2ZMAX,K2Z)
	K1=DSQRT(K1X**2+K1Y**2+K1Z**2)
	K2=DSQRT(K2X**2+K2Y**2+K2Z**2)
	DELTAHE=EBIHE+K1**2/2D0
	DELTAHEP=EBIHEP+K2**2/2D0
	QZ1=DELTAHE/VI
	QZ2=DELTAHEP/VI
! FDCS calculations
! SI of He
	STEP=1
	FDCSHEU=FDCSDITS2(ZP,VI,THEORY,STEP,
     *   	QP1,PHIQ,K1X,K1Y,K1Z)
      FDCSHEN=FDCSHEU/FDCSHEMAX
! SI of He+      
	STEP=2
	FDCSHEPU=FDCSDITS2(ZP,VI,THEORY,STEP,
     *   	QP2,PHIQ,K2X,K2Y,K2Z)    
      FDCSHEPN=FDCSHEPU/FDCSHEPMAX
      IF(FDCSHEN.GT.U1.AND.FDCSHEPN.GT.U2)THEN
! Only file selection (forced when parallel implementation is used)
	IF(SCREEN.EQ.0)THEN
	WRITE(7,110) K1X,K1Y,K1Z,K2X,K2Y,K2Z,QP1,QP2
	ELSE
! Only screen
	IF(SCREEN.EQ.1)THEN
	WRITE(*,110) K1X,K1Y,K1Z,K2X,K2Y,K2Z,QP1,QP2
	ELSE
	WRITE(*,*) 'Option not supported'
	STOP
	ENDIF
	ENDIF
	ENDIF
      ENDDO
      ELSE
	WRITE(*,*) 'Theory not supported'
	STOP      
	ENDIF
	ENDIF
	ENDIF
	

100   FORMAT(7F12.8)
110   FORMAT(8F12.8)

	CLOSE(7)	       
	END PROGRAM
        
! SUBROUTINE RIF TS1
	  
      DOUBLE COMPLEX FUNCTION RIFTS1DI(ZP,VI,QP,PHIQ,
     *	K1X,K1Y,K1Z,K2X,K2Y,K2Z)
	IMPLICIT NONE
	REAL*8 V,K1,K2,QP,PHIQ,K1X,K1Y,K1Z,K2X,K2Y,K2Z
	REAL*8 QV,QMIN,QM,PW3,NORM
	REAL*8 PI,EPS,VI,K1T,K2T,Q1T,Q2T,Q1TM,Q2TM,K1TM,K2TM
	REAL*8 ZP,ZTF,EBI,DELTAE,ZA,ZB,DP
	REAL*8 ALPHA1T,ALPHA2T,NORMA,NORMB
	COMPLEX*16 JP,JT,RIFDI
	COMPLEX*16 C1,CI,C0,COULF1T,COULF2T,CGAMMA
	COMPLEX*16 M0_1A,M0_1B,M0_2A,M0_2B
	COMPLEX*16 M1_1A,M1_1B,M1_2A,M1_2B
      DIMENSION QV(3),Q1T(3),Q2T(3),V(3)
      DIMENSION K1T(3),K2T(3),K1TM(3),K2TM(3)
	PARAMETER(C1=(1D0,0D0),CI=(0D0,1D0),C0=(0D0,0D0))
	PARAMETER(PI=3.1415926535897932d0,EPS=1D-8)
! Helium DI
	EBI=2.904D0
	ZTF=1.6875D0
! INITIAL DATA
	V(1)=0D0
	V(2)=0D0
	V(3)=VI
! FINAL DATA
! ELECTRON MOMENTA WITH RESPECT TO THE TARGET
! Electron 1
      K1T(1)=K1X
	K1T(2)=K1Y
	K1T(3)=K1Z
	K1=DSQRT(K1X**2+K1Y**2+K1Z**2)
! Electron 2
	K2T(1)=K2X
	K2T(2)=K2Y
	K2T(3)=K2Z
	K2=DSQRT(K2X**2+K2Y**2+K2Z**2)
	DELTAE=(EBI+(K1**2)/2D0+(K2**2)/2D0)
	QMIN=DELTAE/VI
! MOMENTUM TRANSFER
      QV(1)=QP*DCOS(PHIQ)
	QV(2)=QP*DSIN(PHIQ)
	QV(3)=QMIN
	QM=DSQRT(QV(1)*QV(1)+QV(2)*QV(2)+QV(3)*QV(3))

	Q1T(1)=QV(1)-K1T(1)
	Q1T(2)=QV(2)-K1T(2)
	Q1T(3)=QV(3)-K1T(3)
	Q1TM=DSQRT(Q1T(1)*Q1T(1)+Q1T(2)*Q1T(2)+Q1T(3)*Q1T(3))

	Q2T(1)=QV(1)-K2T(1)
	Q2T(2)=QV(2)-K2T(2)
	Q2T(3)=QV(3)-K2T(3)
	Q2TM=DSQRT(Q2T(1)*Q2T(1)+Q2T(2)*Q2T(2)+Q2T(3)*Q2T(3))
! INTEGRALS
! SOMMERFERLD PARAMETERS
	IF(K1.LT.1D-2)THEN
	K1=1D-2
	ENDIF
	IF(K2.LT.1D-2)THEN
	K2=1D-2
	ENDIF
	ALPHA1T=(ZTF/K1)
	ALPHA2T=(ZTF/K2)
! NORMALIZATION FACTORS
	COULF1T=CGAMMA(C1-CI*ALPHA1T)*DEXP(PI*ALPHA1T/2D0)
	COULF2T=CGAMMA(C1-CI*ALPHA2T)*DEXP(PI*ALPHA2T/2D0)
	PW3=1D0/(DSQRT(2D0*PI)**9)
! PROJECTILE VARIABLES
      DP=QM**2+EPS**2
! Projectile integrals
	JP=-4D0*PI*ZP/DP
! Target Variables
	ZA=2.183171d0
	ZB=1.188530d0
! Needed for the g77 compiler
      K1TM(1)=-K1X
	K1TM(2)=-K1Y
	K1TM(3)=-K1Z
	
	K2TM(1)=-K2X
	K2TM(2)=-K2Y
	K2TM(3)=-K2Z	
	CALL L0_INTEGRAL(ZA,ALPHA1T,K1T,K1TM,M0_1A)
	CALL L0_INTEGRAL(ZB,ALPHA1T,K1T,K1TM,M0_1B)
	CALL L0_INTEGRAL(ZA,ALPHA2T,K2T,K2TM,M0_2A)
	CALL L0_INTEGRAL(ZB,ALPHA2T,K2T,K2TM,M0_2B)

	CALL L0_INTEGRAL(ZA,ALPHA1T,K1T,Q1T,M1_1A)
	CALL L0_INTEGRAL(ZB,ALPHA1T,K1T,Q1T,M1_1B)
	CALL L0_INTEGRAL(ZA,ALPHA2T,K2T,Q2T,M1_2A)
	CALL L0_INTEGRAL(ZB,ALPHA2T,K2T,Q2T,M1_2B)

	JT=ZTF*(M0_1A*M0_2B+M0_1B*M0_2A)-
     * (M1_1A*M0_2B+M1_1B*M0_2A+M1_2A*M0_1B+M1_2B*M0_1A)
! Composition of the transition matrix 
	RIFDI=C0
	NORMA=(2D0*ZA)**(3D0/2D0)/DSQRT(8D0*PI)
	NORMB=(2D0*ZB)**(3D0/2D0)/DSQRT(8D0*PI)
	NORM=NORMA*NORMB
      RIFDI=RIFDI+NORM*JT*JP

	RIFTS1DI=RIFDI*PW3*COULF1T*COULF2T
	RETURN
	END

! Calculation of the FDCS
! TS1
	DOUBLE PRECISION FUNCTION FDCSDITS1(ZP,VI,
     *	QP,PHIQ,K1X,K1Y,K1Z,K2X,K2Y,K2Z)
	IMPLICIT NONE
	REAL*8 ZP,VI,QP,PHIQ,K1X,K1Y,K1Z,K2X,K2Y,K2Z
	COMPLEX*16 RIFTS1DI
	EXTERNAL RIFTS1DI
! FDCS for DI in TS1
	FDCSDITS1=QP*
     *  CDABS(RIFTS1DI(ZP,VI,QP,PHIQ,K1X,K1Y,K1Z,K2X,K2Y,K2Z))**2
	RETURN
	END

! FDCS in TS2
	DOUBLE PRECISION FUNCTION FDCSDITS2(ZP,VI,THEORY,STEP,
     *   	QP,PHIQ,KX,KY,KZ)
	IMPLICIT NONE
	REAL*8 ZP,VI,QP,PHIQ,KX,KY,KZ
	INTEGER THEORY,STEP
	COMPLEX*16 RIFFBA,RIFCDWEIS
	EXTERNAL RIFFBA,RIFCDWEIS
! TS2 SI steps
! FBA
	IF(THEORY.EQ.1)THEN
	FDCSDITS2=QP*
     *	CDABS(RIFFBA(ZP,VI,STEP,QP,PHIQ,KX,KY,KZ))**2
      ELSE
! CDW-EIS      
	IF(THEORY.EQ.2)THEN
	FDCSDITS2=QP*
     *	CDABS(RIFCDWEIS(ZP,VI,STEP,QP,PHIQ,KX,KY,KZ))**2
      ENDIF
      ENDIF
	RETURN
	END

! SUBROUTINE SI STEP
! RIF in FBA  
      DOUBLE COMPLEX FUNCTION RIFFBA(ZP,VI,STEP,QP,PHIQ,
     *	KX,KY,KZ)
	IMPLICIT NONE
	REAL*8 V,KT,K,QP,PHIQ,KP,KPM,KX,KY,KZ
	REAL*8 QV,QMIN,QM,QT,QTM,ALPHAT,PW3,NORM
	REAL*8 PI,EPS,VI
	REAL*8 ZP,ZTF,ZTEFF,EBI,DELTAE
	INTEGER STEP
	COMPLEX*16 DP,DT,ST,UT,AT,BT
	COMPLEX*16 JP,JT,RIF
	COMPLEX*16 C1,CI,C0,COULFT,CGAMMA
      DIMENSION KT(3),KP(3),QV(3),QT(3),V(3)
	PARAMETER(C1=(1D0,0D0),CI=(0D0,1D0),C0=(0D0,0D0))
	PARAMETER(PI=3.1415926535897932d0,EPS=1D-8)
! Target selection
! SI of He
	IF(STEP.EQ.1)THEN
	EBI=0.91D0
	ZTF=1.3549D0
	ZTEFF=1.3549D0
	ELSE
! SI of He+	
	IF(STEP.EQ.2)THEN
	EBI=2D0
	ZTF=2D0
	ZTEFF=2D0
	ENDIF
	ENDIF
! INITIAL DATA
	V(1)=0D0
	V(2)=0D0
	V(3)=VI
! FINAL DATA
! ELECTRON MOMENTA WITH RESPECT TO THE TARGET
      KT(1)=KX
	KT(2)=KY
	KT(3)=KZ
	K=DSQRT(KX**2+KY**2+KZ**2)
	DELTAE=(EBI+(K**2)/2D0)
	QMIN=DELTAE/VI
! ELECTRON MOMENTA WITH RESPECT TO THE PROJECTILE
      KP(1)=KT(1)-V(1)
	KP(2)=KT(2)-V(2)
	KP(3)=KT(3)-V(3)
	KPM=DSQRT(KP(1)*KP(1)+KP(2)*KP(2)+KP(3)*KP(3))
! MOMENTUM TRANSFER
      QV(1)=QP*DCOS(PHIQ)
	QV(2)=QP*DSIN(PHIQ)
	QV(3)=QMIN
	QM=DSQRT(QV(1)*QV(1)+QV(2)*QV(2)+QV(3)*QV(3))
	QT(1)=QV(1)-KT(1)
	QT(2)=QV(2)-KT(2)
	QT(3)=QV(3)-KT(3)
	QTM=DSQRT(QT(1)*QT(1)+QT(2)*QT(2)+QT(3)*QT(3))
! Sommerfeld parameters
	ALPHAT=(ZTF/K)
! Normalization factors
	COULFT=CGAMMA(C1-CI*ALPHAT)*DEXP(PI*ALPHAT/2D0)
	PW3=1D0/(DSQRT(2D0*PI)**9)      
! PROJECTILE VARIABLES
      DP=QM**2+EPS**2
! Projectile integrals
	JP=-4D0*PI*ZP/DP
! TARGET VARIABLES
	DT=ZTEFF**2+QTM**2
	ST=QT(1)*KT(1)+QT(2)*KT(2)+QT(3)*KT(3)-CI*ZTEFF*K
	UT=2D0*ST/DT
	AT=C1+UT
	BT=2D0*(CI*K+ZTEFF*UT)
! TARGET INTEGRALS
      JT=(4D0*PI*CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))*
     * (2D0*ZTEFF-CI*ALPHAT*BT/AT)
! Composition of the transition matrix 
	RIF=C0
	NORM=DSQRT(ZTEFF**3/PI)
      RIF=RIF+NORM*JT*JP
	RIFFBA=RIF*PW3*COULFT
	RETURN
	END

! RIF in CDW-EIS PRIOR  
      DOUBLE COMPLEX FUNCTION RIFCDWEIS(ZP,VI,STEP,QP,PHIQ,
     *	KX,KY,KZ)
	IMPLICIT NONE
	REAL*8 V,KT,K,QP,PHIQ,KP,KPM,KX,KY,KZ
	REAL*8 QV,QMIN,QM,QT,QTM,ALPHAI,ALPHAP,ALPHAT,PW3,NORM
	REAL*8 PI,EPS,VI
	REAL*8 ZP,ZTF,ZTEFF,EBI,DELTAE
	COMPLEX*16 DP,S1P,S2P,S3P,U1P,U2P,U3P,A1P,A2P,A3P,X1
	COMPLEX*16 DT,ST,UT,AT,BT
	COMPLEX*16 J91,J92,J93,JP,IP,JT,IT,RIF
	COMPLEX*16 C1,CI,C0,COULFI,COULFP,COULFT
	COMPLEX*16 F1,F1PLUS,F1PLUS1,CGAMMA,CGAMMAI
	INTEGER STEP
	DIMENSION KT(3),KP(3),QV(3),QT(3),V(3)
	DIMENSION JT(3),JP(3)
	PARAMETER(C1=(1D0,0D0),CI=(0D0,1D0),C0=(0D0,0D0))
	PARAMETER(PI=3.1415926535897932D0,EPS=1D-8)

! Target selection
! SI of He
	IF(STEP.EQ.1)THEN
	EBI=0.91D0
	ZTF=1.3549D0
	ZTEFF=1.3549D0
	ELSE

! SI of He+
	IF(STEP.EQ.2)THEN
	EBI=2D0
	ZTF=2D0
	ZTEFF=2D0
	ENDIF
	ENDIF

! Initial data
	V(1)=0D0
	V(2)=0D0
	V(3)=VI

! Final data
! Electron momentum with respect to the target
      KT(1)=KX
	KT(2)=KY
	KT(3)=KZ
	K=DSQRT(KX**2+KY**2+KZ**2)
	DELTAE=(EBI+(K**2)/2D0)
	QMIN=DELTAE/VI

! Electron momentum with respect to the projectile
      KP(1)=KT(1)-V(1)
	KP(2)=KT(2)-V(2)
	KP(3)=KT(3)-V(3)
	KPM=DSQRT(KP(1)*KP(1)+KP(2)*KP(2)+KP(3)*KP(3))
! Momentum transfer
      QV(1)=QP*DCOS(PHIQ)
	QV(2)=QP*DSIN(PHIQ)
	QV(3)=QMIN
	QM=DSQRT(QV(1)*QV(1)+QV(2)*QV(2)+QV(3)*QV(3))
	QT(1)=QV(1)-KT(1)
	QT(2)=QV(2)-KT(2)
	QT(3)=QV(3)-KT(3)
	QTM=DSQRT(QT(1)*QT(1)+QT(2)*QT(2)+QT(3)*QT(3))
! Sommerfeld parameters
      ALPHAI=(ZP/VI)
	ALPHAP=(ZP/KPM)
	ALPHAT=(ZTF/K)
! Normalization factors
      COULFI=CGAMMA(C1-CI*ALPHAI)*DEXP(PI*ALPHAI/2D0)
	COULFP=CGAMMA(C1-CI*ALPHAP)*DEXP(PI*ALPHAP/2D0)
	COULFT=CGAMMA(C1-CI*ALPHAT)*DEXP(PI*ALPHAT/2D0)
	PW3=1D0/(DSQRT(2D0*PI)**9)
! Integrals	      
! Projectile Variables
      DP=QM**2+EPS**2
	S1P=-(QV(1)*V(1)+QV(2)*V(2)+QV(3)*V(3))-CI*EPS*VI
	S2P=-(QV(1)*KP(1)+QV(2)*KP(2)+QV(3)*KP(3))-CI*EPS*KPM
	S3P=VI*KPM-(KP(1)*V(1)+KP(2)*V(2)+KP(3)*V(3))
      U1P=(2D0*S1P/DP)
	U2P=(2D0*S2P/DP)
	U3P=(2D0*S3P/DP)
      A1P=(C1+U1P)
	A2P=(C1+U2P)
	A3P=(C1+U3P)
	X1=(C1-((U1P-U3P)/(U1P*A2P)))
      CALL CF21D(ALPHAI*CI,ALPHAP*CI,C1,X1,EPS,F1)
      CALL CF21D(ALPHAI*CI+C1,ALPHAP*CI+C1,C1+C1,X1,EPS,F1PLUS)
      CALL CF21D(ALPHAI*CI+C1,ALPHAP*CI,C1,X1,EPS,F1PLUS1)
	J91=CI*EPS*F1+
     * (CI*ALPHAP/A2P)*F1PLUS*(KPM+CI*EPS*U3P/U1P)
	J92=-(CI*ALPHAP*VI/A2P)*F1PLUS
	J93=-CI*ALPHAP*VI*U3P*F1PLUS/(U1P*A2P)-VI*F1
! Vectorial integral
      JP(1)=8D0*PI*COULFI*CI*ALPHAI*(1D0/(DP**2))*
     *CDEXP(-(CI*ALPHAI+C1)*
     * CDLOG(U1P))*CDEXP(-CI*ALPHAP*CDLOG(A2P))*
     * (J91*V(1)+J92*KP(1)-J93*QV(1))
        
      JP(2)=8D0*PI*COULFI*CI*ALPHAI*(1D0/(DP**2))*
     * CDEXP(-(CI*ALPHAI+C1)*
     * CDLOG(U1P))*CDEXP(-CI*ALPHAP*CDLOG(A2P))* 
     * (J91*V(2)+J92*KP(2)-J93*QV(2))   
     
      JP(3)=8D0*PI*COULFI*CI*ALPHAI*(1D0/(DP**2))*
     * CDEXP(-(CI*ALPHAI+C1)*
     * CDLOG(U1P))*CDEXP(-CI*ALPHAP*CDLOG(A2P))*
     * (J91*V(3)+J92*KP(3)-J93*QV(3))   
! Scalar integral
      CGAMMAI=-CI*CGAMMA(-CI*ALPHAI)*DEXP(PI*ALPHAI/2D0)
      IP=(4D0*PI*VI*(ALPHAI**2D0)*CGAMMAI*
     * CDEXP(-(CI*ALPHAI+C1)*CDLOG(U1P))*
     * CDEXP(-CI*ALPHAP*CDLOG(A2P))*F1PLUS1)/DP

! Target variables
	DT=ZTEFF**2+QTM**2
	ST=QT(1)*KT(1)+QT(2)*KT(2)+QT(3)*KT(3)-CI*ZTEFF*K
	UT=2D0*ST/DT
	AT=C1+UT
	BT=2D0*(CI*K+ZTEFF*UT)
! Target integrals
! Vectorial integral
	JT(1)=-ZTEFF*(8D0*PI*CI*
     * CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))* 
     *(CI*ALPHAT*KT(1)/AT+(C1-CI*ALPHAT+(CI*ALPHAT/AT))*QT(1))

	JT(2)=-ZTEFF*(8D0*PI*CI*
     * CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))* 
     *(CI*ALPHAT*KT(2)/AT+(C1-CI*ALPHAT+(CI*ALPHAT/AT))*QT(2))

	JT(3)=-ZTEFF*(8D0*PI*CI*
     * CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))* 
     *(CI*ALPHAT*KT(3)/AT+(C1-CI*ALPHAT+(CI*ALPHAT/AT))*QT(3))
! Scalar integral
      IT=(4D0*PI*CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))*
     * (2D0*ZTEFF-CI*ALPHAT*BT/AT)

! Composition of the transition matrix 
	RIF=C0
	NORM=DSQRT(ZTEFF**3/PI)
      RIF=RIF+NORM
     * *(JT(1)*JP(1)+JT(2)*JP(2)+JT(3)*JP(3)-IT*IP)
	RIFCDWEIS=RIF*PW3*COULFT*COULFP
	RETURN
	END

	
! Subroutine for generation of random numbers between
! arbritary limits (use RAN2 of Numerical Recipes)
	SUBROUTINE RND(IDUM,XMIN,XMAX,X)
	IMPLICIT NONE
	INTEGER IDUM
	REAL*8 XMIN,XMAX,X
	REAL RAN2
      X=RAN2(IDUM)*(XMAX-XMIN)+XMIN
	RETURN
	END

! Subroutine for searching the maximum (heuristic algorithm
! based on HPSORT of Numerical Recipes)
! TS-1 model
	SUBROUTINE FDCSDITS1M(ZP,VI,QMAX,FDCSDITS1MAX)
	IMPLICIT NONE
	REAL*8 K1MIN,K2MIN,ZP,VI,QP,PHIQ,TOL,QMAX,EMIN,RY
	INTEGER I,NMAX
	REAL*8 FDCSDII,DELTAQ,FDCSDITS1,FDCSDITS1MAX,FDCSDITS1LIM 
	DIMENSION FDCSDII(10000)
	EXTERNAL FDCSDITS1
	PARAMETER(TOL=5D-4,RY=27.21D0)
	EMIN=.1D0
	PHIQ=0D0
	K1MIN=DSQRT(2D0*EMIN/RY)
	K2MIN=DSQRT(2D0*EMIN/RY)
	NMAX=QMAX/TOL
	IF(NMAX.GT.10000)THEN
	WRITE(*,*) 'Increase Tol'
	STOP
	ENDIF
! Array building
	DELTAQ=QMAX/NMAX
	DO I=1,NMAX
	QP=(I-1)*DELTAQ
	FDCSDII(I)=FDCSDITS1(ZP,VI,
     *	QP,PHIQ,0D0,0D0,K1MIN,0D0,0D0,K2MIN)
	ENDDO
! Array sorting		
	CALL HPSORT(NMAX,FDCSDII)
! Verify that the maximum is not the FDCS at Qmax
	FDCSDITS1LIM=FDCSDITS1(ZP,VI,
     *	QMAX,PHIQ,0D0,0D0,K1MIN,0D0,0D0,K2MIN)
	FDCSDITS1MAX=FDCSDII(NMAX)
	IF(FDCSDITS1LIM.EQ.FDCSDITS1MAX)THEN
	WRITE(*,*) 'Maximum of FDCS out of range. Extend Qmax'
	STOP
	ENDIF
	RETURN
	END

! Subroutine for searching the maximum (heuristic algorithm
! based on HPSORT of Numerical Recipes)
! TS-2 model
	SUBROUTINE FDCSTS2M(ZP,VI,QMAX,STEP,THEORY,FDCSDIMAX)
	IMPLICIT NONE
	REAL*8 KMIN,ZP,VI,QP,PHIQ,TOL,QMAX,EMIN,RY
	INTEGER I,NMAX,STEP,THEORY
	REAL*8 FDCSDII,DELTAQ,FDCSDIMAX,FDCSDILIM,FDCSDITS2
	DIMENSION FDCSDII(10000)
	EXTERNAL FDCSDITS2
	PARAMETER(TOL=5D-4,RY=27.21D0)
	EMIN=.1D0
	PHIQ=0D0
	KMIN=DSQRT(2D0*EMIN/RY)
	NMAX=QMAX/TOL
	IF(NMAX.GT.10000)THEN
	WRITE(*,*) 'Increase Tol'
	STOP
	ENDIF
 ! Array building
	DELTAQ=QMAX/NMAX
	DO I=1,NMAX
	QP=(I-1)*DELTAQ
	FDCSDII(I)=FDCSDITS2(ZP,VI,THEORY,STEP,
     *   	QP,PHIQ,0D0,0D0,KMIN)
	ENDDO
 ! Array sorting		
	CALL HPSORT(NMAX,FDCSDII)
 ! Verify that the maximum is not the FDCS at Qmax
	FDCSDILIM=FDCSDITS2(ZP,VI,THEORY,STEP,
     *   	QMAX,PHIQ,0D0,0D0,KMIN)
	FDCSDIMAX=FDCSDII(NMAX)
	IF(FDCSDILIM.EQ.FDCSDIMAX)THEN
	WRITE(*,*) 'Maximum of FDCS out of range. Extend Qmax'
	STOP
	ENDIF
	RETURN
	END
