! MCEG Driver program
! 
!***************************************************************!
!  This program first asks the user to choose between           !
!  the processes supported by the MCEG.                         !
!  The user should provide the files 'si.dat' or 'di.dat'       !
!  depending of his/her selection with the format described     !
!  in the article                                               !
!  The program verifies that all the parameters are correct     !                                                !
!                                                               !
! NOTE: Please check that the MC samples be less than 2**31-1   !
! in order to avoid an out of range error in the file loading   ! 
!***************************************************************!

	PROGRAM MCEG
	IMPLICIT NONE
	INTEGER OPTION
	REAL*8 ZP,EI,QMAX
	REAL*8 KXMIN,KXMAX,KYMIN,KYMAX,KZMIN,KZMAX
	REAL*8 K1XMIN,K1XMAX,K1YMIN,K1YMAX,K1ZMIN,K1ZMAX
	REAL*8 K2XMIN,K2XMAX,K2YMIN,K2YMAX,K2ZMIN,K2ZMAX
	INTEGER TARGET,THEORY,SEED,EEMODEL,SCREEN
	INTEGER*4 MCPOINTS
	REAL*8 V_C,VI,RY,MP
	PARAMETER(V_C=137d0,RY=27.21D0,MP=1836.D0)
	  
	WRITE(*,*) 'Monte Carlo Event Generator program for'
	WRITE(*,*) 'the generation of event files in single and'
	WRITE(*,*) 'double ionization of atoms by ion impact'
	WRITE(*,*) '****************************************'
	WRITE(*,*) ''
	WRITE(*,*) 'Please choose the process'
	WRITE(*,*) '0 = Single Ionization'
	WRITE(*,*) '1 = Double Ionization'
	READ(*,*) OPTION
	
	IF(OPTION.EQ.0)THEN
	WRITE(*,*) 'Single Ionization'
	WRITE(*,*) 'Please remember to prepare a file named "si.dat"
     * with the required format'
      GO TO 10
	ELSE
	IF(OPTION.EQ.1)THEN
	WRITE(*,*) 'Double Ionization'
	WRITE(*,*) 'Please remember to prepare a file named "di.dat"
     * with the required format'
      GO TO 20
	ELSE
	WRITE(*,*) 'Process not supported'
	STOP
	ENDIF
	ENDIF
	  
! Parameters reading for SI
	  
10    WRITE(*,*) ''
	WRITE(*,*) 'Single Ionization file reading'
	WRITE(*,*) ''
	OPEN(UNIT=9,FILE='si.dat',status='unknown')
	READ(9,*) ZP
	READ(9,*) EI
	READ(9,*) TARGET
	READ(9,*) THEORY
	READ(9,*) KXMIN, KXMAX
	READ(9,*) KYMIN, KYMAX
	READ(9,*) KZMIN, KZMAX	  
	READ(9,*) QMAX
	READ(9,*) MCPOINTS
	READ(9,*) SCREEN
	READ(9,*) SEED
	CLOSE(9)
	  	
! Some checks of the parameters are necessary

	IF(ZP.LE.0d0)THEN
	WRITE(*,*) 'Projectile not supported'
	STOP
	ELSE
	IF(ZP.GT.92d0)THEN
	WRITE(*,*) 'ZP>92 does not exist!!!'
	STOP	  
	ENDIF
	ENDIF

	IF(EI.LE.0d0)THEN
	WRITE(*,*) 'Projectile energy must be greater than 0!!!'
	STOP
	ENDIF

	IF(TARGET.EQ.0)THEN
	WRITE(*,*) 'You have chosen Hydrogen as a target'
	GO TO 30
	ELSE
	IF(TARGET.EQ.1)THEN
	WRITE(*,*) 'You have chosen He as a target'
	GO TO 30
	ELSE
	WRITE(*,*) 'Target atom not supported'
	STOP
	ENDIF
	ENDIF
	  
30	IF(THEORY.EQ.0)THEN
	WRITE(*,*) 'You have chosen FBA'
	GO TO 40
	ELSE
	IF(THEORY.EQ.1)THEN
	WRITE(*,*) 'You have chosen CDW-EIS'
	GO TO 40
	ELSE
	WRITE(*,*) 'Theory not supported'
	STOP
	ENDIF
	ENDIF

40	IF(KXMIN.GE.KXMAX)THEN
	WRITE(*,*) 'Kxmin must be smaller than Kxmax'
	STOP
	ENDIF

      IF(KYMIN.GE.KYMAX)THEN
	WRITE(*,*) 'Kymin must be smaller than Kymax'
	STOP
	ENDIF

      IF(KZMIN.GE.KZMAX)THEN
	WRITE(*,*) 'Kzmin must be smaller than Kzmax'
	STOP
	ENDIF

	IF(QMAX.LE.0d0)THEN
	WRITE(*,*) 'Qmax must be greater than 0!!!'
	STOP
	ENDIF
	  
	IF(MCPOINTS.LE.0)THEN
	WRITE(*,*) 'The number of MC points must be greater than 0!!!'
	STOP
	ENDIF

	IF(SEED.EQ.0)THEN
	WRITE(*,*) 'Running in a parallel environment'
	WRITE(*,*) 'The screen option will be forced to 0'
	SCREEN=0
	WRITE(*,*) 'Remember to provide the seed using the'
	WRITE(*,*) 'corresponding environment variable' 	
	WRITE(*,*) '(see the article for details)'	  
	CONTINUE
	ELSE
	WRITE(*,*) 'Running in serial'
	CONTINUE
      ENDIF	  
	  	  
	IF(SCREEN.EQ.0)THEN
	WRITE(*,*) 'Only file results'
	CONTINUE
	ELSE
	IF(SCREEN.EQ.1)THEN
	WRITE(*,*) 'Only screen results'
	CONTINUE
	ELSE
	WRITE(*,*) 'Check screen option'
	STOP
	ENDIF
	ENDIF



! Projectile Velocity calculation (in a.u. and relativistically corrected)

      VI=V_C*DSQRT((1D0-(EI*1000D0/(RY*MP*V_C**2)+1D0)**(-2)))      

! Print a summary of the parameters

	WRITE(*,*) ' '
	WRITE(*,*) 'Summary of parameters SI'
	WRITE(*,*) ' '
	WRITE(*,100) 'Projectile charge:',ZP
	WRITE(*,110) 'Projectile energy:',EI,' keV/amu'
	WRITE(*,120) 'Velocity: ',VI,' a.u.'
	WRITE(*,*) 'Electron momentum ranges'
	WRITE(*,130) KXMIN,' <= Kx <=',KXMAX
	WRITE(*,130) KYMIN,' <= Ky <=',KYMAX
	WRITE(*,130) KZMIN,' <= Kz <=',KZMAX
	WRITE(*,*) 'Momentum transfer range'
      WRITE(*,140) '0 <= QX <=',QMAX
	WRITE(*,150) 'MC samples: ',MCPOINTS
	WRITE(*,*) 'Ready to run'
	WRITE(*,*) 'Parameters checked'

	IF(SEED.EQ.0)THEN
	WRITE(*,*) 'Use"qsub [job array options] mceg_si" (see your'
	WRITE(*,*) 'queue system manual for details)'
	ELSE
	WRITE(*,*) 'Use "mceg_si"'
	ENDIF
	STOP
	    	  
      STOP

! Parameters reading for DI	  

20    WRITE(*,*) ''
      WRITE(*,*) 'Double Ionization file reading'
	WRITE(*,*) ''
	OPEN(UNIT=9,FILE='di.dat',status='unknown')
	READ(9,*) ZP
	READ(9,*) EI
	READ(9,*) THEORY
	READ(9,*) EEMODEL
	READ(9,*) K1XMIN, K1XMAX
	READ(9,*) K1YMIN, K1YMAX
	READ(9,*) K1ZMIN, K1ZMAX
	READ(9,*) K2XMIN, K2XMAX
	READ(9,*) K2YMIN, K2YMAX
	READ(9,*) K2ZMIN, K2ZMAX
	READ(9,*) QMAX
	READ(9,*) MCPOINTS
	READ(9,*) SCREEN
	READ(9,*) SEED
	CLOSE(9)

! Some checks of the parameters are necessary

	IF(ZP.LE.0d0)THEN
	WRITE(*,*) 'Projectile not supported'
	STOP
	ELSE
	IF(ZP.GT.92d0)THEN
	WRITE(*,*) 'ZP>92 does not exist!!!'
	STOP	  
	ENDIF
	ENDIF

	IF(EI.LE.0d0)THEN
	WRITE(*,*) 'Projectile energy must be greater than 0!!!'
	STOP
	ENDIF

 	IF(THEORY.EQ.0)THEN
	WRITE(*,*) 'You have chosen the TS-1 model' 
	WRITE(*,*) '(based on FBA)'
	GO TO 50
	ELSE
	IF(THEORY.EQ.1)THEN
	WRITE(*,*) 'You have chosen the TS-2 simulation'
	WRITE(*,*) '(based on FBA)'
	WRITE(*,*) 'the e-e model will be ignored for this model'
	GO TO 60
	ELSE
	IF(THEORY.EQ.2)THEN
	WRITE(*,*) 'You have chosen the TS-2 simulation'
	WRITE(*,*) '(based on CDW-EIS)'
	WRITE(*,*) 'the e-e model will be ignored for this model'
	GO TO 60
	ELSE
	WRITE(*,*) 'Theory not supported'
	STOP
	ENDIF
	ENDIF
	ENDIF

50 	IF(EEMODEL.EQ.0)THEN
	WRITE(*,*) 'You have chosen the uncorrelated model for'
	WRITE(*,*) 'the e-e interaction'
	GO TO 60
	ELSE
	IF(EEMODEL.EQ.1)THEN
	WRITE(*,*) 'You have chosen the static screening model for'
	WRITE(*,*) 'the e-e interaction'
	GO TO 60
	ELSE
	IF(EEMODEL.EQ.2)THEN
	WRITE(*,*) 'You have chosen the dynamic screening model for'
	WRITE(*,*) 'the e-e interaction'
	GO TO 60
	ELSE	  
	WRITE(*,*) 'Theory for e-e interaction not supported'
	STOP
	ENDIF
	ENDIF
	ENDIF

60	IF(K1XMIN.GE.K1XMAX)THEN
	WRITE(*,*) 'K1xmin must be smaller than K1xmax'
	STOP
	ENDIF

      IF(K1YMIN.GE.K1YMAX)THEN
	WRITE(*,*) 'K1ymin must be smaller than K1ymax'
	STOP
	ENDIF

      IF(K1ZMIN.GE.K1ZMAX)THEN
	WRITE(*,*) 'K1zmin must be smaller than K1zmax'
	STOP
	ENDIF

	IF(K2XMIN.GE.K2XMAX)THEN
	WRITE(*,*) 'K2xmin must be smaller than K2xmax'
	STOP
	ENDIF

      IF(K2YMIN.GE.K2YMAX)THEN
	WRITE(*,*) 'K2ymin must be smaller than K2ymax'
	STOP
	ENDIF

      IF(K2ZMIN.GE.K2ZMAX)THEN
	WRITE(*,*) 'K1zmin must be smaller than K1zmax'
	STOP
	ENDIF
	  
	IF(QMAX.LE.0d0)THEN
	WRITE(*,*) 'Qmax must be greater than 0!!!'
	STOP
	ENDIF
	  
	IF(MCPOINTS.LE.0)THEN
	WRITE(*,*) 'The number of MC points must be greater than 0!!!'
	STOP
	ENDIF

	IF(SEED.EQ.0)THEN
	WRITE(*,*) 'Running in a parallel environment'
	WRITE(*,*) 'The screen option will be forced to 0'
	SCREEN=0
	WRITE(*,*) 'Remember to provide the seed using the'
	WRITE(*,*) 'corresponding environment variable' 	
	WRITE(*,*) '(see the article for details)'	  
	CONTINUE
	ELSE
	WRITE(*,*) 'Running in serial'
	CONTINUE
      ENDIF	  
		  	  
	IF(SCREEN.EQ.0)THEN
	WRITE(*,*) 'Only file results'
	CONTINUE
	ELSE
	IF(SCREEN.EQ.1)THEN
	WRITE(*,*) 'Only screen results'
	CONTINUE
	ELSE
	WRITE(*,*) 'Check screen option'
	STOP
	ENDIF
	ENDIF

! Projectile Velocity calculation (in a.u. and relativistically corrected)

      VI=V_C*DSQRT((1D0-(EI*1000D0/(RY*MP*V_C**2)+1D0)**(-2)))      

! Print a summary of the parameters

      WRITE(*,*) ' '
	WRITE(*,*) 'Summary of parameters DI'
	WRITE(*,*) ' '
	WRITE(*,100) 'Projectile charge:',ZP
	WRITE(*,110) 'Projectile energy:',EI,' keV/amu'
	WRITE(*,120) 'Velocity: ',VI,' a.u.'
	WRITE(*,*) 'Electron 1 momentum ranges'
	WRITE(*,130) K1XMIN,' <= K1x <= ',K1XMAX
	WRITE(*,130) K1YMIN,' <= K1y <= ',K1YMAX
	WRITE(*,130) K1ZMIN,' <= K1z <= ',K1ZMAX
	WRITE(*,*) 'Electron 2 momentum ranges'
	WRITE(*,130) K2XMIN,' <= K2x <= ',K2XMAX
	WRITE(*,130) K2YMIN,' <= K2y <= ',K2YMAX
	WRITE(*,130) K2ZMIN,' <= K2z <= ',K2ZMAX
	WRITE(*,*) 'Momentum transfer range'
      WRITE(*,140) '0 <= QX <=',QMAX
	WRITE(*,150) 'MC samples: ',MCPOINTS
	WRITE(*,*) 'Ready to run'
	WRITE(*,*) 'Parameters checked'
	
	IF(SEED.EQ.0)THEN
	WRITE(*,*) 'Use"qsub [job array options] mceg_di" (see your'
	WRITE(*,*) 'queue system manual for details)'
	ELSE
	WRITE(*,*) 'Use "mceg_di"'
	ENDIF
	STOP
	  
100   FORMAT(A18,1F6.2)
110   FORMAT(A18,1F10.2,A4) 
120   FORMAT(A10,1F5.2,A5)
130   FORMAT(1F5.2,A11,1F5.2) 
140   FORMAT(A10,1F5.2)
150   FORMAT(A12,I10)    
      
	END PROGRAM
	
