! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
! 
!  HYPERGEOMETRIC FUNCTION 2F1(CA,CB,CC,CX), COMPLEX DOUBLE PRECISION
! 
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      SUBROUTINE CF21D(CA,CB,CC,CX,ER,CF)
      IMPLICIT REAL*8(A-B,D-H,O-Z), COMPLEX*16(C)
      PARAMETER(C1=(1.D0,0.D0))

      IF (CA.EQ.CB) GO TO 2
1     CALL CF21C(CA,CB,CC,CX,ER,CF)
      RETURN

2     X=CDABS(CX)
      IF (X.LT.1.D0) GO TO 3

!     TO AVOIDE PROBLEMS OF CONTINUATION WHEN CA=CB
!     WE USE GRADS-RYZH. EQ 9.131/1

      CBN=CC-CB
      CXN=CX/(CX-C1)
      CALL CF21C(CA,CBN,CC,CXN,ER,CF)
      CF=CF*CDEXP(-CA*CDLOG(C1-CX))
      RETURN

3     IF(X.LT.0.75D0) GO TO 4
      CALL CF21B(CA,CB,CC,CX,ER,CF)
      RETURN

4     CALL CF21A(CA,CB,CC,CX,ER,CF)
      RETURN
      END
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      SUBROUTINE CF21C(CA,CB,CC,CX,ER,CF)
      IMPLICIT REAL*8(A-B,D-H,O-Z), COMPLEX*16(C)
      PARAMETER(C0=(0.D0,0.D0), C1=(1.D0,0.D0))

!     INTRODUCTION

      CF=C1
      IF(CA.EQ.C0.OR.CB.EQ.C0) RETURN
      IF(CX.EQ.C0) RETURN
      IF(CA.EQ.CC) GO TO 20
      IF(CB.EQ.CC) GO TO 30
      GO TO 40
20    CF=CDEXP(-CB*CDLOG(C1-CX))
      RETURN
30    CF=CDEXP(-CA*CDLOG(C1-CX))
      RETURN
40    IF(CC.EQ.C0) STOP '1. CF21C, CC=0'

      X=CDABS(CX)
      IF(X.GT.1.D0) GO TO 1
      IF(X.LT.0.75D0) GO TO 2

      CALL CF21B(CA,CB,CC,CX,ER,CF)
      RETURN

1     CONTINUE
      CX2=CX/(CX-C1)
      CX3=C1-CX
      X2=CDABS(CX2)
      X3=CDABS(CX3)
      IF(X3.LT.0.1D0) GO TO 11
      IF(X2.LT.0.75D0) GO TO 12
      GO TO 13

!     GRADS-RYZH. EQ. 9.131/1

12    CONTINUE
      CCA=CC-CA
      CCB=CC-CB
      HCA=CDABS(CCA*CB)
      HCB=CDABS(CCB*CA)
      IF(HCB.LT.HCA) GO TO 21
      CALL CF21A(CB,CCA,CC,CX2,ER,CF)
      CF=CF*CDEXP(-CB*CDLOG(CX3))
      RETURN
21    CALL CF21A(CA,CCB,CC,CX2,ER,CF)
      CF=CF*CDEXP(-CA*CDLOG(CX3))
      RETURN

!     GRADS-RYZH. EQ 9.131/2

11    CCAB=CC-CA-CB
      IF(CCAB.EQ.C0) GO TO 13
      CCA=CC-CA
      CCB=CC-CB
      CC1=C1-CCAB
      CC2=C1+CCAB
      CF1=CDEXP(CLGAM(CCAB)-CLGAM(CCA)-CLGAM(CCB))
      CF2=CDEXP(CLGAM(-CCAB)-CLGAM(CA)-CLGAM(CB)+CCAB*CDLOG(CX3))
      CALL CF21A(CA,CB,CC1,CX3,0.5D0*ER,CH1)
      CALL CF21A(CCA,CCB,CC2,CX3,0.5D0*ER,CH2)
      CF=(CF1*CH1+CF2*CH2)*CGAMMA(CC)
      RETURN
13    CONTINUE

!     THE ANALYTICAL CONTINUATION IS PERFORMED TO
!     REFRAIN /CX/<1. GRADS-RYZH. EQ 9.132/2

      IF(CA.EQ.CB) STOP 'CF21C, CA=CB, CONT. IMPOSSIBLE'
      CZ=C1/CX
      CAUX=CDLOG(-CX)
      CAUX1=-CA*CAUX
      CAUX2=-CB*CAUX
      CBA=CB-CA
      CCA=CC-CA
      CCB=CC-CB
      CB1=C1-CCA
      CC1=C1-CBA
      CB2=C1-CCB
      CC2=C1+CBA
      CALL CF21B(CA,CB1,CC1,CZ,0.5D0*ER,CF1)
      CF1=CF1*CDEXP(CAUX1)*CGAMMA(CBA)/(CGAMMA(CCA)*CGAMMA(CB))
      CALL CF21B(CB,CB2,CC2,CZ,0.5D0*ER,CF2)
      CF2=CF2*CDEXP(CAUX2)*CGAMMA(-CBA)/(CGAMMA(CCB)*CGAMMA(CA))
      CF=(CF1+CF2)*CGAMMA(CC)
      RETURN

!     IT DIRECTLY USES THE CF21A

2     CALL CF21A(CA,CB,CC,CX,ER,CF)
      RETURN
      END
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!     CF21B USES THE CF21A IN THE MORE PROFITABLE WAY
!     HERE IT IS SUPPOSED THAT ALREADY /CX/<1
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      SUBROUTINE CF21B(CA,CB,CC,CX,ER,CF)
      IMPLICIT REAL*8(A-B,D-H,O-Z), COMPLEX*16(C)
      PARAMETER(C0=(0.D0,0.D0), C1=(1.D0,0.D0))

!     INTRODUCTION

      CF=C1
      IF(CA.EQ.C0.OR.CB.EQ.C0) RETURN
      IF(CX.EQ.C0) RETURN
      IF(CA.EQ.CC) GO TO 20
      IF(CB.EQ.CC) GO TO 30
      GO TO 40
20    CF=CDEXP(-CB*CDLOG(C1-CX))
      RETURN
30    CF=CDEXP(-CA*CDLOG(C1-CX))
      RETURN
40    IF(CC.EQ.C0) STOP '2. CF21C,CC=0'

!     IT CONSIDERS THREE POSSIBLE FORMULA

      X=CDABS(CX)
      IF(X.GT.1.D0) STOP 'CF21B, X>1'
      CX2=CX/(CX-C1)
      CX3=C1-CX
      X2=CDABS(CX2)
      X3=CDABS(CX3)
      IF(X3.LT.0.1D0) GO TO 3
4     IF(X2.LT.X) GO TO 2

!     THE USUAL EXPRESSION GRADS-RYZH. EQ. 9.100

1     CALL CF21A(CA,CB,CC,CX,ER,CF)
      RETURN

!     GRADS-RYZH. EQ. 9.131/1

2     CONTINUE
      CCA=CC-CA
      CCB=CC-CB
      HCA=CDABS(CCA*CB)
      HCB=CDABS(CCB*CA)
      IF(HCB.LT.HCA) GO TO 21
      CALL CF21A(CB,CCA,CC,CX2,ER,CF)
      CF=CF*CDEXP(-CB*CDLOG(CX3))
      RETURN
21    CALL CF21A(CA,CCB,CC,CX2,ER,CF)
      CF=CF*CDEXP(-CA*CDLOG(CX3))
      RETURN

!     GRADS-RYZH. EQ. 9.131/2

3     CCAB=CC-CA-CB
      IF(CCAB.EQ.C0) GO TO 4
      CCA=CC-CA
      CCB=CC-CB
      CC1=C1-CCAB
      CC2=C1+CCAB
      CF1=CGAMMA(CCAB)/(CGAMMA(CCA)*CGAMMA(CCB))
      CF2=CGAMMA(-CCAB)*CDEXP(CCAB*CDLOG(CX3))/(CGAMMA(CA)*CGAMMA(CB))
      CALL CF21A(CA,CB,CC1,CX3,0.5D0*ER,CH1)
      CALL CF21A(CCA,CCB,CC2,CX3,0.5D0*ER,CH2)
      CF=(CF1*CH1+CF2*CH2)*CGAMMA(cc)
      RETURN
      END
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  >>>>>.
!     THE CF21A CALCULATES THE HYPERGEOMETRIC WITH THE USUAL
!     EXPANSION GRADS-RYZH. EQ. 9.100, BUT THE POWER SERIES
!     IS TRANSFORMED IN A CONTINUED FRACTION TO IMPROVE PRECESSION
!     ABRAMOWITZ-STEGUN 3.30.1(6),WHICH IS SUMMED IN THE USUAL
!     FORM ABRAMOWITZ-STEGUN 3.30.1(1,2,3)
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      SUBROUTINE CF21A(CA,CB,CC,CX,ER,CF)
      IMPLICIT REAL*8(A-B,D-H,O-Z), COMPLEX*16(C)
      PARAMETER(C0=(0.D0,0.D0), C1=(1.D0,0.D0))
      PARAMETER(ZERO=0.D0, ONE=1.D0)
      CF=C1
      IF(CA.EQ.C0.OR.CB.EQ.C0) RETURN
      IF(CX.EQ.C0) RETURN
      IF(CA.EQ.CC) GO TO 20
      IF(CB.EQ.CC) GO TO 30
      GO TO 40
20    CF=CDEXP(-CB*CDLOG(C1-CX))
      RETURN
30    CF=CDEXP(-CA*CDLOG(C1-CX))
      RETURN
40    IF(CC.EQ.C0) then
	  WRITE(*,*) '3. CF21C, CC=0'
      RETURN
      END IF
      CN1=C0
      CN2=C1
      CD1=C1
      CD2=C1
      CF2=C1
      F1=ONE
      F2=ONE
      R1=ONE
      R2=ONE
      RM2=ZERO

1     CONTINUE
      RM1=RM2+ONE
      CE=(CA+RM2)*(CB+RM2)*CX/((CC+RM2)*RM1)
      CAX=-CE
      CBX=CE+C1
      CN3=CAX*CN1+CBX*CN2
      CD3=CAX*CD1+CBX*CD2
      CF3=CN3/CD3
      F3=CDABS(CF3)
      IF(F3.NE.ZERO) GO TO 3
      IF(F1.EQ.ZERO.AND.F2.EQ.ZERO) GO TO 2
      R3=ONE
      GO TO 4

3     R3=CDABS(CF2-CF3)/F3
4     R=DMAX1(R1,R2,R3)
      IF(R.LT.ER) GO TO 2
      CF2=CF3
      CN1=CN2
      CN2=CN3
      CD1=CD2
      CD2=CD3
      F1=F2
      F2=F3
      R1=R2
      R2=R3
      RM2=RM1
      GO TO 1

2     CF=CF3
      RETURN
      END


! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!     CGAMMA(C)= GAMMA FUNCTION (C)                     .
!               COMPLEX DOUBLE PRECISION
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      COMPLEX*16 FUNCTION CGAMMA(C)
      IMPLICIT REAL*8(A-B,D-H,O-Z), COMPLEX*16(C)
	  DIMENSION H(6)
      DATA H/1.D0,+5.772157D-01,-6.558781D-01,-4.2002635D-02,
     *	+1.665386D-01,-4.219773D-02/
      DATA ER/1.D-12/
      XC=CDABS(C)
      IF(XC.LT.0.01D0) GOTO 1

2     DR=XC*ER
      DI=DR
      CALL CLGAMA(C,0,DR,DI,IPOLO,CLGAM)
	CGAMMA=CDEXP(CLGAM)
      RETURN

1     IF(XC.LT.ER) GOTO 2
      CGAM=C*(H(1)+C*(H(2)+C*(H(3)+C*(H(4)+C*(H(5)+C*H(6))))))
      CGAMMA=1.D0/CGAM

      RETURN
      END

! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!     CLGAM(C)= LOGARITHM OF THE GAMMA FUNCTION (C)                     .
!               COMPLEX DOUBLE PRECISION
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      FUNCTION CLGAM(C)
      IMPLICIT REAL*8(A-B,D-H,O-Z), COMPLEX*16(C)
      DIMENSION H(6)
      DATA H/1.D0,+5.772157D-01,-6.558781D-01,-4.2002635D-02,
     *	+1.665386D-01,-4.219773D-02/
      DATA ER/1.D-12/
      XC=CDABS(C)
      IF(XC.LT.0.01D0) GOTO 1

2     DR=XC*ER
      DI=DR
      CALL CLGAMA(C,0,DR,DI,IPOLO,CLGAM)
      RETURN

1     IF(XC.LT.ER) GOTO 2
      CGAM=C*(H(1)+C*(H(2)+C*(H(3)+C*(H(4)+C*(H(5)+C*H(6))))))
      CGAM=1.D0/CGAM
      CLGAM=CDLOG(CGAM)
      RETURN
      END
! >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      SUBROUTINE CLGAMA(ZZ,IWRITE,DR,DI,IPOLO,LOGAM)
      IMPLICIT REAL*8(A-H,O-Z)
      COMPLEX*16 Z,Z2,LOGAM,C,ZA,ZZ
      COMPLEX*16 C0,C1,R,W
      DIMENSION A(2),B(12),G(7)
      EQUIVALENCE (ZA,A(1))
      PARAMETER(C0=(0.D0,0.D0), C1=(1.D0,0.D0))
      PARAMETER(NZERO=0, NONE=1, NSIX=6)
      PARAMETER(ZERO=0.D0, ONE=1.D0, TWO=2.D0, FOUR=4.D0)
      PARAMETER(A0=9.189385D-1, A8=0.5D0)
      DATA B/+8.333333D-2,-2.777778D-3,+7.936508D-4,	  
     *       -5.952381D-4,+8.417508D-4,-1.917527D-3,	  
     *      +6.410256D-3,-2.955065D-2,+1.796444D-1,	  
     *       -1.392432D+0,+1.340286D+1,-1.568483D+2/
      DATA G/+8.333333D-2,+3.333333D-2,+2.523809D-1,	
     *       +5.256065D-1,+1.011523D+0,+1.517474D+0,	
     *       +2.269489D+0/

      ZA=ZZ
      Z=ZZ
      LOGAM=C0
      IRZ=INT(A(1))
      D=TWO*DR
      IF(A(1)-D) 2,2,1
2     DM=DABS(A(1)-DFLOAT(IRZ))
      IF(DM.GT.DR) GOTO 1
      IF(DABS(A(2)).GT.DI) GOTO 1
      WRITE(*,3) Z
3     FORMAT(2X,'Z=',2(1PD14.7,2X),'IS CONSIDERED AS A POLE')

      IPOLE=NONE
      RETURN

1     CONTINUE
      IPOLE=NZERO
      IF(A(1).GE.ONE) GOTO 20
      NG=NSIX-IRZ
      IF(NG) 4,4,5

5     C=CMPLX(DFLOAT(NG),ZERO)
      Z=Z+C
4     Z2=Z*Z
      R=ONE/Z2
      LOGAM=(B(1)+R*(B(2)+R*(B(3)+R*(B(4)+R*(B(5)+R*(B(6)+R*(B(7)+R*  
     *     (B(8)+R*(B(9)+R*B(10))))))))))/Z

8     LOGAM=LOGAM+A0-Z+(Z-A8)*CDLOG(Z)
      IF(NG) 100,100,9

9     C=C1
      DO 10 I=1,NG
      Z=Z-C
      LOGAM=LOGAM-CDLOG(Z)

10    CONTINUE
      GOTO 100

20    CONTINUE
      LOGAM=(Z-A8)*CDLOG(Z)+A0-Z
      LOGAM=LOGAM+G(1)/(Z+G(2)/(Z+G(3)/(Z+G(4)/(Z+G(5)/(Z+G(6)/  
     *           (Z+G(7)/Z))))))

100   CONTINUE
      IF(IWRITE.EQ.1) WRITE(5,200) ZZ,LOGAM
200   FORMAT(2X,'Z=',2(1PD14.7,2X),5X,'LOG GAMMA(Z)=',2(1PD14.7,2X))
      RETURN
      END
      
