! Monte Carlo Event Generator for SI
! of H and He for ion impact     
      PROGRAM MCEG_SI
! Uncomment if you use Intel Fortran Compilers
!      USE IFLPORT
	IMPLICIT NONE
	REAL*8 MP,VI,EPS,V_C,PI,ZP,EI
	REAL*8 RY,DELTAE,EBI,QMIN,QP,PHIQ
	REAL*8 FDCSR,FDCS,FDCSMAX,FDCSN
	REAL*8 KX,KY,KZ,K,QMAX,U
	REAL*8 KXMIN,KXMAX,KYMIN,KYMAX,KZMIN,KZMAX
	INTEGER TARGET,THEORY,MC,MCPOINTS,SCREEN
	INTEGER ISEED,ITASK
	CHARACTER*3  TASK_ID
	EXTERNAL FDCS
	PARAMETER(PI=3.1415926535897932D0,MP=1836D0,EPS=1D-6)
	PARAMETER(V_C=137D0,RY=27.21D0)
! File load (check if your system requires the full path)
	OPEN(UNIT=9,FILE='si.dat',STATUS='UNKNOWN')
	READ(9,*) ZP
	READ(9,*) EI
	READ(9,*) TARGET
	READ(9,*) THEORY
	READ(9,*) KXMIN, KXMAX
	READ(9,*) KYMIN, KYMAX
	READ(9,*) KZMIN, KZMAX	  
	READ(9,*) QMAX
	READ(9,*) MCPOINTS
	READ(9,*) SCREEN
	READ(9,*) ISEED
	CLOSE(9)	
! Initial projectile velocity
      VI=V_C*DSQRT((1D0-(EI*1000D0/(RY*MP*V_C**2)+1D0)**(-2)))
! Target selection
	IF(TARGET.EQ.0)THEN
! Hydrogen
	EBI=0.5D0
	ELSE
	IF(TARGET.EQ.1)THEN
! Helium
	EBI=0.91D0
	ELSE
	WRITE(*,*) 'Target not supported'
	STOP
	ENDIF
	ENDIF
! Parallel implementation
	IF(ISEED.EQ.0)THEN
! SGE
	CALL GETENV("SGE_TASK_ID",TASK_ID)
! PBS Torque
!	CALL GETENV("PBS_ARRAYID",TASK_ID)
	READ(TASK_ID,'(I3)') ITASK
	SCREEN=0
	ENDIF
! Test if only screen or file output
	IF(SCREEN.EQ.0)THEN
	IF(ISEED.EQ.0)THEN
! Check if your system needs the full path		
	OPEN(7,FILE='results.dat.'//TASK_ID,
     *  STATUS='UNKNOWN')
	ISEED=ITASK
	ELSE
	OPEN(7,FILE='results.dat',
     *  STATUS='UNKNOWN')
	ENDIF
	ENDIF
! Some initialization
	FDCSR=0D0
	FDCSN=0D0
	FDCSMAX=0D0
	PHIQ=0D0
! Maximum value of FDCS calculation
	CALL FDCSM(ZP,VI,TARGET,THEORY,QMAX,FDCSMAX)
! MC main loop
	DO MC=1,MCPOINTS
	CALL RND(ISEED,0D0,1D0,U)
	CALL RND(ISEED,0D0,QMAX,QP)
	CALL RND(ISEED,KXMIN,KXMAX,KX)
	CALL RND(ISEED,KYMIN,KYMAX,KY)
	CALL RND(ISEED,KZMIN,KZMAX,KZ)
	K=DSQRT(KX**2+KY**2+KZ**2)
	DELTAE=(EBI+(K**2)/2D0)
	QMIN=DELTAE/VI
	FDCSR=FDCS(ZP,VI,TARGET,THEORY,QP,PHIQ,KX,KY,KZ)
	FDCSN=FDCSR/FDCSMAX
! MC test
	IF(U.LT.FDCSN)THEN
! Only file selection (forced when parallel implementation is used)
	IF(SCREEN.EQ.0)THEN
	WRITE(7,100) KX,KY,KZ,QP
	ELSE
! Only screen
	IF(SCREEN.EQ.1)THEN
	WRITE(*,100) KX,KY,KZ,QP
	ELSE
	WRITE(*,*) 'Option not supported'
	STOP
	ENDIF
	ENDIF
	ENDIF
	ENDDO

100   FORMAT(4F12.8)
	CLOSE(7)	       
	END PROGRAM
        
! SUBROUTINE RIF CDW-EIS PRIOR
	  
      DOUBLE COMPLEX FUNCTION RIFCDWEIS(ZP,VI,TARGET,QP,PHIQ,
     *	KX,KY,KZ)
	IMPLICIT NONE
	REAL*8 V,KT,K,QP,PHIQ,KP,KPM,KX,KY,KZ
	REAL*8 QV,QMIN,QM,QT,QTM,ALPHAI,ALPHAP,ALPHAT,PW3,NORM
	REAL*8 PI,EPS,LAMBDA,A,VI
	REAL*8 ZP,ZTF,EBI,DELTAE
	COMPLEX*16 DP,S1P,S2P,S3P,U1P,U2P,U3P,A1P,A2P,A3P,X1
	COMPLEX*16 DT,ST,UT,AT,BT
	COMPLEX*16 J91,J92,J93,JP,IP,JT,IT,RIF
	COMPLEX*16 C1,CI,C0,COULFI,COULFP,COULFT
	COMPLEX*16 F1,F1PLUS,F1PLUS1,CGAMMA,CGAMMAI
	INTEGER TARGET,I,IMAX
	DIMENSION LAMBDA(5),A(5)
	DIMENSION KT(3),KP(3),QV(3),QT(3),V(3)
	DIMENSION JT(3,5),JP(3),IT(5)
! He HF model
      DATA LAMBDA/1.41714D0,2.37682D0,4.39628D0,6.52699D0,7.94252D0/
      DATA A/0.76838D0,0.22346D0,0.04082D0,-0.00994D0,0.00230D0/
    
	PARAMETER(C1=(1D0,0D0),CI=(0D0,1D0),C0=(0D0,0D0))
	PARAMETER(PI=3.1415926535897932D0,EPS=1D-8)

! Target selection
	IF(TARGET.EQ.0)THEN

! Hydrogen
	EBI=0.5D0
	ZTF=1D0
	IMAX=1
	A(1)=1D0
	LAMBDA(1)=1D0
	ELSE
	IF(TARGET.EQ.1)THEN

! Helium
	EBI=0.91D0
	ZTF=1.6875D0
	IMAX=5
	ELSE
	WRITE(*,*) 'Target not supported'
	STOP
	ENDIF
	ENDIF

! Initial data
	V(1)=0D0
	V(2)=0D0
	V(3)=VI

! Final data
! Electron momentum with respect to the target
      KT(1)=KX
	KT(2)=KY
	KT(3)=KZ
	K=DSQRT(KX**2+KY**2+KZ**2)
	DELTAE=(EBI+(K**2)/2D0)
	QMIN=DELTAE/VI

! Electron momentum with respect to the projectile
      KP(1)=KT(1)-V(1)
	KP(2)=KT(2)-V(2)
	KP(3)=KT(3)-V(3)
	KPM=DSQRT(KP(1)*KP(1)+KP(2)*KP(2)+KP(3)*KP(3))
! Transfer momentum
      QV(1)=QP*DCOS(PHIQ)
	QV(2)=QP*DSIN(PHIQ)
	QV(3)=QMIN
	QM=DSQRT(QV(1)*QV(1)+QV(2)*QV(2)+QV(3)*QV(3))
	QT(1)=QV(1)-KT(1)
	QT(2)=QV(2)-KT(2)
	QT(3)=QV(3)-KT(3)
	QTM=DSQRT(QT(1)*QT(1)+QT(2)*QT(2)+QT(3)*QT(3))
! Sommerfeld parameters
      ALPHAI=(ZP/VI)
	ALPHAP=(ZP/KPM)
	ALPHAT=(ZTF/K)
! Normalization factors
      COULFI=CGAMMA(C1-CI*ALPHAI)*DEXP(PI*ALPHAI/2D0)
	COULFP=CGAMMA(C1-CI*ALPHAP)*DEXP(PI*ALPHAP/2D0)
	COULFT=CGAMMA(C1-CI*ALPHAT)*DEXP(PI*ALPHAT/2D0)
	PW3=1D0/(DSQRT(2D0*PI)**9)
! Integrals	      
! Projectile Variables
      DP=QM**2+EPS**2
	S1P=-(QV(1)*V(1)+QV(2)*V(2)+QV(3)*V(3))-CI*EPS*VI
	S2P=-(QV(1)*KP(1)+QV(2)*KP(2)+QV(3)*KP(3))-CI*EPS*KPM
	S3P=VI*KPM-(KP(1)*V(1)+KP(2)*V(2)+KP(3)*V(3))
      U1P=(2D0*S1P/DP)
	U2P=(2D0*S2P/DP)
	U3P=(2D0*S3P/DP)
      A1P=(C1+U1P)
	A2P=(C1+U2P)
	A3P=(C1+U3P)
	X1=(C1-((U1P-U3P)/(U1P*A2P)))
      CALL CF21D(ALPHAI*CI,ALPHAP*CI,C1,X1,EPS,F1)
      CALL CF21D(ALPHAI*CI+C1,ALPHAP*CI+C1,C1+C1,X1,EPS,F1PLUS)
      CALL CF21D(ALPHAI*CI+C1,ALPHAP*CI,C1,X1,EPS,F1PLUS1)
	J91=CI*EPS*F1+
     * (CI*ALPHAP/A2P)*F1PLUS*(KPM+CI*EPS*U3P/U1P)
	J92=-(CI*ALPHAP*VI/A2P)*F1PLUS
	J93=-CI*ALPHAP*VI*U3P*F1PLUS/(U1P*A2P)-VI*F1
! Vectorial integral
      JP(1)=8D0*PI*COULFI*CI*ALPHAI*(1D0/(DP**2))*
     *CDEXP(-(CI*ALPHAI+C1)*
     * CDLOG(U1P))*CDEXP(-CI*ALPHAP*CDLOG(A2P))*
     * (J91*V(1)+J92*KP(1)-J93*QV(1))
        
      JP(2)=8D0*PI*COULFI*CI*ALPHAI*(1D0/(DP**2))*
     * CDEXP(-(CI*ALPHAI+C1)*
     * CDLOG(U1P))*CDEXP(-CI*ALPHAP*CDLOG(A2P))* 
     * (J91*V(2)+J92*KP(2)-J93*QV(2))   
     
      JP(3)=8D0*PI*COULFI*CI*ALPHAI*(1D0/(DP**2))*
     * CDEXP(-(CI*ALPHAI+C1)*
     * CDLOG(U1P))*CDEXP(-CI*ALPHAP*CDLOG(A2P))*
     * (J91*V(3)+J92*KP(3)-J93*QV(3))   
! Scalar integral
      CGAMMAI=-CI*CGAMMA(-CI*ALPHAI)*DEXP(PI*ALPHAI/2D0)
      IP=(4D0*PI*VI*(ALPHAI**2D0)*CGAMMAI*
     * CDEXP(-(CI*ALPHAI+C1)*CDLOG(U1P))*
     * CDEXP(-CI*ALPHAP*CDLOG(A2P))*F1PLUS1)/DP

! Target variables
	DO I=1,IMAX
	DT=LAMBDA(I)**2+QTM**2
	ST=QT(1)*KT(1)+QT(2)*KT(2)+QT(3)*KT(3)-CI*LAMBDA(I)*K
	UT=2D0*ST/DT
	AT=C1+UT
	BT=2D0*(CI*K+LAMBDA(I)*UT)
! Target integrals
! Vectorial integral
	JT(1,I)=-LAMBDA(I)*(8D0*PI*CI*
     * CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))* 
     *(CI*ALPHAT*KT(1)/AT+(C1-CI*ALPHAT+(CI*ALPHAT/AT))*QT(1))

	JT(2,I)=-LAMBDA(I)*(8D0*PI*CI*
     * CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))* 
     *(CI*ALPHAT*KT(2)/AT+(C1-CI*ALPHAT+(CI*ALPHAT/AT))*QT(2))

	JT(3,I)=-LAMBDA(I)*(8D0*PI*CI*
     * CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))* 
     *(CI*ALPHAT*KT(3)/AT+(C1-CI*ALPHAT+(CI*ALPHAT/AT))*QT(3))
! Scalar integral
      IT(I)=(4D0*PI*CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))*
     * (2D0*LAMBDA(I)-CI*ALPHAT*BT/AT)
	ENDDO

! Transition matrix building
	RIF=C0
	DO I=1,IMAX
	NORM=A(I)*(2D0*LAMBDA(I))**(3D0/2D0)/DSQRT(8D0*PI)
      RIF=RIF+NORM
     * *(JT(1,I)*JP(1)+JT(2,I)*JP(2)+JT(3,I)*JP(3)-IT(I)*IP)
	ENDDO
	RIFCDWEIS=RIF*PW3*COULFT*COULFP
	RETURN
	END

! SUBROUTINE RIF FBA
	  
      DOUBLE COMPLEX FUNCTION RIFFBA(ZP,VI,TARGET,QP,PHIQ,
     *	KX,KY,KZ)
	IMPLICIT NONE
	REAL*8 V,KT,K,QP,PHIQ,KP,KPM,KX,KY,KZ
	REAL*8 QV,QMIN,QM,QT,QTM,ALPHAT,PW3,NORM
	REAL*8 PI,EPS,LAMBDA,A,VI
	REAL*8 ZP,ZTF,EBI,DELTAE
	COMPLEX*16 DP,DT,ST,UT,AT,BT
	COMPLEX*16 JP,JT,RIF
	COMPLEX*16 C1,CI,C0,COULFT,CGAMMA
	INTEGER TARGET,I,IMAX
       DIMENSION KT(3),KP(3),QV(3),QT(3),V(3),JT(5)
	DIMENSION LAMBDA(5),A(5)
! He HF model
      DATA LAMBDA/1.41714D0,2.37682D0,4.39628D0,6.52699D0,7.94252D0/
      DATA A/0.76838D0,0.22346D0,0.04082D0,-0.00994D0,0.00230D0/
	PARAMETER(C1=(1D0,0D0),CI=(0D0,1D0),C0=(0D0,0D0))
	PARAMETER(PI=3.1415926535897932d0,EPS=1D-8)

! Target selection
	IF(TARGET.EQ.0)THEN
! Hydrogen
	EBI=0.5D0
	ZTF=1D0
	IMAX=1
	A(1)=1D0
	LAMBDA(1)=1D0
	ELSE
	IF(TARGET.EQ.1)THEN
! Helium
	EBI=0.91D0
	ZTF=1.6875D0
	IMAX=5
	ELSE
	WRITE(*,*) 'Target not supported'
	STOP
	ENDIF
	ENDIF
! INITIAL DATA
	V(1)=0D0
	V(2)=0D0
	V(3)=VI
! FINAL DATA
! ELECTRON MOMENTA WITH RESPECT TO THE TARGET
      KT(1)=KX
	KT(2)=KY
	KT(3)=KZ
	K=DSQRT(KX**2+KY**2+KZ**2)
	DELTAE=(EBI+(K**2)/2D0)
	QMIN=DELTAE/VI
! ELECTRON MOMENTA WITH RESPECT TO THE PROJECTILE
      KP(1)=KT(1)-V(1)
	KP(2)=KT(2)-V(2)
	KP(3)=KT(3)-V(3)
	KPM=DSQRT(KP(1)*KP(1)+KP(2)*KP(2)+KP(3)*KP(3))
! MOMENTUM TRANSFER 
      QV(1)=QP*DCOS(PHIQ)
	QV(2)=QP*DSIN(PHIQ)
	QV(3)=QMIN
	QM=DSQRT(QV(1)*QV(1)+QV(2)*QV(2)+QV(3)*QV(3))
	QT(1)=QV(1)-KT(1)
	QT(2)=QV(2)-KT(2)
	QT(3)=QV(3)-KT(3)
	QTM=DSQRT(QT(1)*QT(1)+QT(2)*QT(2)+QT(3)*QT(3))
! Sommerfeld parameters
	ALPHAT=(ZTF/K)
! Normalization factors
	COULFT=CGAMMA(C1-CI*ALPHAT)*DEXP(PI*ALPHAT/2D0)
	PW3=1D0/(DSQRT(2D0*PI)**9)      
! PROJECTILE VARIABLES
      DP=QM**2+EPS**2
! Projectile integrals
	JP=-4D0*PI*ZP/DP
! TARGET VARIABLES
	DO I=1,IMAX
	DT=LAMBDA(I)**2+QTM**2
	ST=QT(1)*KT(1)+QT(2)*KT(2)+QT(3)*KT(3)-CI*LAMBDA(I)*K
	UT=2D0*ST/DT
	AT=C1+UT
	BT=2D0*(CI*K+LAMBDA(I)*UT)
! TARGET INTEGRALS
      JT(I)=(4D0*PI*CDEXP(-CI*ALPHAT*CDLOG(AT))/(DT**2))*
     * (2D0*LAMBDA(I)-CI*ALPHAT*BT/AT)
	ENDDO

! Composition of the transition matrix 
	RIF=C0
	DO I=1,IMAX
	NORM=A(I)*(2D0*LAMBDA(I))**(3D0/2D0)/DSQRT(8D0*PI)
      RIF=RIF+NORM*JT(I)*JP
	ENDDO

	RIFFBA=RIF*PW3*COULFT
	RETURN
	END

! Calculation of the FDCS

	DOUBLE PRECISION FUNCTION FDCS(ZP,VI,
     *	TARGET,THEORY,QP,PHIQ,KX,KY,KZ)
	IMPLICIT NONE
	REAL*8 ZP,VI,QP,PHIQ,KX,KY,KZ
	COMPLEX*16 RIFFBA,RIFCDWEIS
	INTEGER THEORY,TARGET

	IF(THEORY.EQ.0)THEN
	FDCS=QP*
     *	CDABS(RIFFBA(ZP,VI,TARGET,QP,PHIQ,KX,KY,KZ))**2
	ELSE
	IF(THEORY.EQ.1)THEN
	FDCS=QP*
     *	CDABS(RIFCDWEIS(ZP,VI,TARGET,QP,PHIQ,KX,KY,KZ))**2
	ELSE
	WRITE(*,*) 'Theory not suppported'
	STOP
	ENDIF
	ENDIF
	RETURN
	END
! Subroutine for generation of random numbers between
! arbritary limits (use RAN2 of Numerical Recipes)
	SUBROUTINE RND(IDUM,XMIN,XMAX,X)
	IMPLICIT NONE
	INTEGER IDUM
	REAL*8 XMIN,XMAX,X
	REAL RAN2
      X=RAN2(IDUM)*(XMAX-XMIN)+XMIN
	RETURN
	END

! Subroutine for searching the maximum (heuristic algorithm
! based on HPSORT of Numerical Recipes)

	SUBROUTINE FDCSM(ZP,VI,TARGET,THEORY,QMAX,FDCSMAX)
	IMPLICIT NONE
	REAL*8 KMIN,ZP,VI,QP,PHIQ,TOL,QMAX,EMIN,RY
	INTEGER TARGET,THEORY,I,NMAX
	REAL*8 FDCSI,DELTAQ,FDCS,FDCSMAX,FDCSLIM 
	DIMENSION FDCSI(10000)
	PARAMETER(TOL=5D-4,RY=27.21D0)

	EMIN=.1D0
	PHIQ=0D0
	KMIN=DSQRT(2D0*EMIN/RY)
	NMAX=QMAX/TOL
	IF(NMAX.GT.10000)THEN
	WRITE(*,*) 'Increase Tol'
	STOP
	ENDIF
! Array building
	DELTAQ=QMAX/NMAX
	DO I=1,NMAX
	QP=(I-1)*DELTAQ
	FDCSI(I)=FDCS(ZP,VI,
     *	TARGET,THEORY,QP,PHIQ,0D0,0D0,KMIN)
	ENDDO
! Array sorting		
	CALL HPSORT(NMAX,FDCSI)
! Verify that the maximum is not the FDCS at Qmax
	FDCSLIM=FDCS(ZP,VI,
     *	TARGET,THEORY,QMAX,PHIQ,0D0,0D0,KMIN)
	FDCSMAX=FDCSI(NMAX)
	IF(FDCSLIM.EQ.FDCSMAX)THEN
	WRITE(*,*) 'Maximum of FDCS out of range. Extend Qmax'
	STOP
	ENDIF
	RETURN
	END
